<?php
/**
 * @brief		spaciousMembersOnline Task
 * @author		<a href='http://www.invisionpower.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) 2001 - 2016 Invision Power Services, Inc.
 * @license		http://www.invisionpower.com/legal/standards/
 * @package		IPS Community Suite
 * @subpackage	spaciouswidgets
 * @since		06 Mar 2017
 * @version		SVN_VERSION_NUMBER
 */

namespace IPS\spaciouswidgets\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * spaciousMembersOnline Task
 */
class _spaciousMembersOnline extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{

		/* Most Members Online at a Time */

		/* Long story short: unless you have the stats widget up on a page on the front end this */
		/* statistic is NEVER tracked. It also gets calculated on EVERY load of the widget */
		/* IPS should pull that and instead run as task, until they do, this does it for them */
		/* Now set to run every 10 minutes. That widget is a grind on well-trafficed sites though. */

		$mostOnline = json_decode( \IPS\Settings::i()->most_online, TRUE );

		$count = \IPS\Db::i()->select( 'COUNT(*)', 'core_sessions', [ 'login_type<>? and running_time > ?', \IPS\Session\Front::LOGIN_TYPE_SPIDER, \IPS\DateTime::create()->sub( new \DateInterval( 'PT30M' ) )->getTimeStamp() ] )->first();
		if( $count > $mostOnline['count'] )
		{
			$mostOnline = [ 'count' => $count, 'time' => time() ];
			// New method for updating settings circa 4.2.5?
			\IPS\Settings::i()->changeValues( array( 'most_online' => json_encode( $mostOnline ) ) );
		}

		/* Now, let's get the actual member guest bot counts, json encode, and save in settings for  */
		/* the dashboard header bit - piggy backing on this task as it seems more reasonable than creating another. */
		$members['notbots_online'] = $count;
		$members['members_online'] = 0;
		$members['guests_online'] = 0;
		$members['anon_online'] = 0;
		$members['spiders_online'] = 0;
		$members['member_ids'] = '';
		$bots = [];

		$where = [
			[ 'core_sessions.running_time>?', \IPS\DateTime::create()->sub( new \DateInterval( 'PT30M' ) )->getTimeStamp() ]
		];

		$i = 1;
		$just_some_bots = array("adsense", "baidu", "bing", "facebook", "google", "websense", "yahoo", "yandex");

		$online = array();
		$seen   = array();
		foreach( \IPS\Db::i()->select( '*', 'core_sessions', $where, 'running_time DESC' ) as $row )
		{

			// Two things here. Online Users by app for the Spacious Online Users widget and
			// the usual online / guest counts but with my spider-bot stuff.
			// Second half of the online widget stuff after this foreach loop.

			// ON HOLD FOR 4.3 RELEASE

			$key = ( $row['member_id'] ? $row['member_id'] : $row['id'] );

			if ( ! isset( $seen[ $key ] ) )
			{
				$online[ $row['current_appcomponent'] ][ $key ] = $row['id'];
				$seen[ $key ] = true;
			}


			switch ( $row['login_type'] )
			{
				/* Member */
				case \IPS\Session\Front::LOGIN_TYPE_MEMBER:
					if ( $row['member_name'] )
					{
						$members['members_online'] += 1;
						// Don't want 10,000 member ids here...
						if ($i < 57)
						{
							$members['member_ids'] = $members['member_ids'] . $row['member_id'] . ',';
							$i++;
						}

					}
					else
					{
						$members['guests_online'] += 1;
					}
					break;

				/* Delicious Spiders */
				case \IPS\Session\Front::LOGIN_TYPE_SPIDER:
					$members['spiders_online'] += 1;

					// Even though spiders are identified according to an internal array list,
					// the exact spider name keyed there is not saved in sessions; the session is merely flagged as
					// a spider and the uagent set as one of the spider variants i.e. a bot called
					// searchymcsearchy might have more than one uagent. To be certain
					// lets just bring over the key from that spider array.

					$imma_spider = \IPS\Http\Useragent::parse($row['browser']);

					// NOTE: Do we need most of this anymore? Not really but leave be for now.
					// Creating the image file name here to save time later.

					$bots[$row['id']]['spider_name'] = $imma_spider->spider;
					if (in_array($imma_spider->spider, $just_some_bots))
					{
						$bots[$row['id']]['spider_image'] = $imma_spider->spider . ".svg";
					}
					else
					{
						$bots[$row['id']]['spider_image'] = '';
					}
					$bots[$row['id']]['spider_image'] = $imma_spider->spider . ".svg";
					$bots[$row['id']]['ip_address'] = $row['ip_address'];
					$bots[$row['id']]['browser'] = $row['browser'];
					$bots[$row['id']]['uagent_key'] = $row['uagent_key'];

					break;

				/* Anons */
				case \IPS\Session\Front::LOGIN_TYPE_ANONYMOUS:
					$members['anon_online'] += 1;
					$members['member_ids'] = $members['member_ids'] . $row['member_id'] . ',';
					break;

				/* Guest */
				case \IPS\Session\Front::LOGIN_TYPE_GUEST:
				case \IPS\Session\Front::LOGIN_TYPE_INCOMPLETE:
				$members['guests_online'] += 1;
					break;
			}
		}


		$breakdown = [];
		$breakdown_percentages = [];
		foreach ($online as $app => $data)
		{
			if( !\IPS\Application::appIsEnabled( $app ) )
			{
				continue;
			}
			$breakdown[$app] = count( $data );

		}

		$members['member_ids'] = rtrim($members['member_ids'], ",");

		// New method for updating settings circa 4.2.5?
		\IPS\Settings::i()->changeValues( array( 'spacious_members_online' => json_encode( $members ) ) );
		\IPS\Settings::i()->changeValues( array( 'spacious_bots_online' => json_encode( $bots ) ) );
		\IPS\Settings::i()->changeValues( array( 'spacious_online_breakdown' => json_encode( $breakdown ) ) );

		return NULL;
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}