<?php
/**
 * @brief		spaciousPatchCheck Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	spaciouswidgets
 * @since		28 Nov 2017
 */

namespace IPS\spaciouswidgets\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * spaciousPatchCheck Task
 */
class _spaciousPatchCheck extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{

		/* Runs every 12 hours, hits the update url for IPS patches, but is overkill for 3rd Party apps and plugins as that */
		/* is just checking the db for filled update data which is populated by a daily IPS task. */

		/* NOTE: Additional hook(s) on install routines to clear out json settings for this of anything that just got */
		/* updated so the buttons actually clear if needed to. Otherwise they linger for 12 to 24 hours after they were */
		/* used. */

		$apps = 0;
		$plugs = 0;
		$ipsapps = 0;

		/* Cropped version of the IPS update check routine. Just want patches (and only current version patches) if they exist */
		try
		{
			$url = \IPS\Http\Url::ips('updateCheck')->setQueryString( [ 'type' => 'support', 'key' => \IPS\Settings::i()->ipb_reg_number ] );
			if ( \IPS\USE_DEVELOPMENT_BUILDS )
			{
				$url = $url->setQueryString( 'development', 1 );
			}

			$response = $url->request()->get()->decodeJson();

			if ( $response['longversion'] > \IPS\Application::load('core')->long_version )
			{

				if ( $response['version'] != \IPS\Application::load( 'core' )->version )
				{
					$ipsapps = 0;
				}
				elseif ( count( $response['changes'] ) )
				{
					\IPS\Db::i()->update(
						'core_sys_conf_settings', [ 'conf_value' => json_encode( $response ) ],
						[ 'conf_key=?', 'spacious_current_patches' ]
					);

					$ipsapps = 1;
				}
			}

		}
		catch ( \Exception $e ) {}


		if ($ipsapps == 0)
		{
			\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => 'FALSE'  ], [ 'conf_key=?', 'spacious_current_patches' ] );

		}

		/* Applications not IPS */
		foreach ( \IPS\Db::i()->select( '*', 'core_applications', \IPS\Db::i()->in( 'app_directory', \IPS\Application::$ipsApps, TRUE ) ) as $app )
		{
			$hold_me = json_decode($app['app_update_version'], TRUE);
			if ( $hold_me['longversion'] > $app['app_long_version'] )
			{
				$appdata['name'] = $app['app_directory'];
				$appdata['data'] = $hold_me;
				$appupdates[$app['app_id']] = $appdata;
				$apps = 1;
			}
		}

		if ($apps == 1)
		{
			\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => json_encode($appupdates)  ], [ 'conf_key=?', 'spacious_current_apppatches' ] );

		}
		else{
			\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => 'FALSE'  ], [ 'conf_key=?', 'spacious_current_apppatches' ] );
		}

		/* Plugins */
		foreach ( \IPS\Db::i()->select( '*', 'core_plugins') as $plugin )
		{
			$hold_me = json_decode($plugin['plugin_update_check_data'], TRUE);
			if ( $hold_me['longversion'] > $plugin['plugin_version_long'] )
			{
				$plugindata['name'] = $plugin['plugin_name'];
				$plugindata['data'] = $hold_me;
				$plugupdates[$plugin['plugin_id']] = $plugindata;
				$plugs = 1;
			}
		}

		if ($plugs == 1)
		{
			\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => json_encode($plugupdates)  ], [ 'conf_key=?', 'spacious_current_plugupdates' ] );

		}
		else{
			\IPS\Db::i()->update( 'core_sys_conf_settings', [ 'conf_value' => 'FALSE'  ], [ 'conf_key=?', 'spacious_current_plugupdates' ] );
		}


		/* All the checks are now either FALSE or filled with json update data. Wipe settings to reset and we're done. */
		unset( \IPS\Data\Store::i()->settings );
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}